import java.applet.Applet;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Label;
import java.awt.MediaTracker;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.StringTokenizer;
import java.util.Vector;

class Command extends Panel
        implements ActionListener, Runnable
{
    final private String FIRST = "[<";
    final private String FAST_BACK = "<<";
    final private String BACK = "<";
    final private String FORWARD = ">";
    final private String FAST_FORWARD = ">>";
    final private String LAST = ">]";
    final private String AUTO_FORWARD = "Auto >";
	final private String AUTO_STOP = "Stop";
	final private String EDIT = "Edit ";
	final private String VIEW = "View";
	final private String SUBMIT = "Submit";

    private Applet itsApplet;
    private Parser itsParser;
    private Defs   itsDefs;
    private Board  itsBoard;

    private Label  itsLabelMove;
    private Button itsButtonFirst;
    private Button itsButtonFastBack;
    private Button itsButtonBack;
    private Button itsButtonForward;
    private Button itsButtonFastForward;
    private Button itsButtonLast;
	private Button itsButtonAutoForward;
	private Button itsButtonEdit;
	private Button itsButtonSubmit;

    private Thread itsAutoForwardThread;

    public Command(Applet applet, Parser parser, Defs defs, Board board)
    {
        itsApplet = applet;
        itsParser = parser;
        itsDefs   = defs;
        itsBoard  = board;

        setFont(itsDefs.getCommandFont());

        if (defs.isShowCoordinates())
        {
            itsLabelMove = new Label("nnn xxx");
            add(itsLabelMove);
        }

		if (defs.getSize() == Defs.BIG_SIZE)
		{
			add(new Label());
		}

        itsButtonFirst = new Button(FIRST);
        itsButtonFirst.addActionListener(this);
        add(itsButtonFirst);

        itsButtonFastBack = new Button(FAST_BACK);
        itsButtonFastBack.addActionListener(this);
        add(itsButtonFastBack);

        itsButtonBack = new Button(BACK);
        itsButtonBack.addActionListener(this);
        add(itsButtonBack);

        itsButtonForward = new Button(FORWARD);
        itsButtonForward.addActionListener(this);
        add(itsButtonForward);

        itsButtonFastForward = new Button(FAST_FORWARD);
        itsButtonFastForward.addActionListener(this);
        add(itsButtonFastForward);

        itsButtonLast = new Button(LAST);
        itsButtonLast.addActionListener(this);
        add(itsButtonLast);

		if (defs.getSize() == Defs.BIG_SIZE)
		{
			add(new Label());
		}

        itsButtonAutoForward = new Button(AUTO_FORWARD);
        itsButtonAutoForward.addActionListener(this);

		if (defs.getSize() == Defs.BIG_SIZE ||
			itsParser.getUrlUpdateOnlineGame() == null)
		{
	        add(itsButtonAutoForward);
		}

		if (defs.getSize() == Defs.BIG_SIZE)
		{
			add(new Label());
		}

		itsButtonEdit = new Button(EDIT);
		itsButtonEdit.addActionListener(this);
		add(itsButtonEdit);

		itsButtonSubmit = new Button(SUBMIT);
		itsButtonSubmit.addActionListener(this);
		itsButtonSubmit.setEnabled(false);

		if (itsParser.getUrlUpdateOnlineGame() != null)
		{
			add(itsButtonSubmit);
		}
    }

    public void actionPerformed(ActionEvent ev)
    {
        String command = ev.getActionCommand();

        if (command.equals(FIRST))
        {
            itsBoard.move(itsBoard.FIRST);
        }
        else if (command.equals(FAST_BACK))
        {
            itsBoard.move(itsBoard.FAST_BACK);
        }
        else if (command.equals(BACK))
        {
            itsBoard.move(itsBoard.BACK);
        }
        else if (command.equals(FORWARD))
        {
            itsBoard.move(itsBoard.FORWARD);
        }
        else if (command.equals(FAST_FORWARD))
        {
            itsBoard.move(itsBoard.FAST_FORWARD);
        }
        else if (command.equals(LAST))
        {
            itsBoard.move(itsBoard.LAST);
        }
        else if (command.equals(AUTO_FORWARD))
        {
            if (itsAutoForwardThread == null)
            {
                start();
            }
        }
        else if (command.equals(AUTO_STOP))
        {
            if (itsAutoForwardThread != null)
            {
                stop();
            }
        }
		else if (command.equals(EDIT))
		{
			edit();
		}
		else if (command.equals(VIEW))
		{
			view();
		}
        else if (command.equals(SUBMIT))
        {
        	submit();
        }
    }

    public void enableButtons(boolean isFirst, boolean isLast)
    {
        itsButtonFirst.setEnabled(!isFirst);
        itsButtonFastBack.setEnabled(!isFirst);
        itsButtonBack.setEnabled(!isFirst);

        itsButtonForward.setEnabled(!isLast);
        itsButtonFastForward.setEnabled(!isLast);
        itsButtonLast.setEnabled(!isLast);

        itsButtonAutoForward.setEnabled(!isLast);

        itsBoard.requestFocus();
    }

    public void showMove(Move move)
    {
        if (itsDefs.isShowCoordinates())
        {
            itsLabelMove.setText(
                move.getNumber()
                    + "  "
                    + Defs.getCoordinateImage(move.getCol(), move.getRow()));
        }
    }

    private void start()
    {
        itsAutoForwardThread = new Thread(this);
        itsAutoForwardThread.start();

        itsButtonAutoForward.setLabel(AUTO_STOP);
        itsApplet.showStatus("Auto Forward: On");
    }

	public void stop()
	{
		try
		{
			itsAutoForwardThread = null;
			itsButtonAutoForward.setLabel(AUTO_FORWARD);
			itsApplet.showStatus("Auto Forward: Off");
		}
		catch (Exception e)
		{
			// Null pointer exception in case applet is destroyed
			// due to change of web page
		}
	}

	private void edit()
	{
		itsButtonEdit.setLabel(VIEW);
		itsApplet.showStatus("Mode: Edit");
		itsBoard.edit();
		itsButtonSubmit.setEnabled(true);
	}

	private void view()
	{
		itsButtonEdit.setLabel(EDIT);
		itsApplet.showStatus("Mode: View");
		itsBoard.view();
		itsButtonSubmit.setEnabled(false);
	}

	private void submit()
	{
		itsBoard.submit();
		view();
		itsBoard.getOnlineMoves(false);
	}

    public void run()
    {
        while (itsAutoForwardThread != null)
        {
            itsBoard.move(itsBoard.FORWARD);

            if (itsBoard.isLastMove())
            {
                break;
            }

            try
            {
                Thread.sleep(1500);
            }
            catch (InterruptedException e)
            {
                break;
            }
        }

        stop();
    }

	public void destroy()
	{
		if (itsAutoForwardThread != null)
		{
			itsAutoForwardThread.interrupt();
		}
	}
}
	