import java.applet.Applet;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Label;
import java.awt.MediaTracker;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.StringTokenizer;
import java.util.Vector;

class Defs
{
    static final public int SMALL_SIZE = 19;
    static final public int BIG_SIZE   = 32;

    private final String FONT_NAME = "SansSerif";

	private int     itsSize;
	private boolean isShowOneLineComment;
	private boolean isShowMultiLineComment;
    private boolean isShowCoordinates;
    private boolean isShowNumbers;
    private boolean isShowCommands;
    private boolean isShowBoard;

	private Font itsCommentFont;
    private Font itsCoordinateFont;
    private Font itsStoneNumberFont;
    private Font itsTextFont;
    private Font itsCommandFont;

    private Color itsColor;

    private long itsRefreshInterval;

    private static final String COORDINATES = " abcdefghijklmno";

	public Defs(Parser parser)
	{
		itsSize                = parser.getSize();
		isShowOneLineComment   = parser.getShowOneLineComment();
		isShowMultiLineComment = parser.getShowMultiLineComment();
        isShowCoordinates      = parser.getShowCoordinates();
        isShowNumbers          = parser.getShowNumbers();
        isShowCommands         = parser.getShowCommands();
        isShowBoard            = parser.getShowBoard();
        itsColor               = parser.getColor();
		itsRefreshInterval     = parser.getRefreshInterval();

        if (itsSize == BIG_SIZE)
        {
            itsCommentFont     = new Font(FONT_NAME, Font.PLAIN, 14);
            itsCoordinateFont  = new Font(FONT_NAME, Font.BOLD, 16);
            itsStoneNumberFont = new Font(FONT_NAME, Font.BOLD, 12);
            itsTextFont        = new Font(FONT_NAME, Font.BOLD, 18);
            itsCommandFont     = new Font(FONT_NAME, Font.PLAIN, 16);
        }
        else
        {
            itsCommentFont     = new Font(FONT_NAME, Font.PLAIN, 10);
            itsCoordinateFont  = new Font(FONT_NAME, Font.BOLD, 12);
            itsStoneNumberFont = new Font(FONT_NAME, Font.BOLD, 10);
            itsTextFont        = new Font(FONT_NAME, Font.BOLD, 14);
            itsCommandFont     = new Font(FONT_NAME, Font.PLAIN, 12);
        }
    }

    static private String getRowImage(int row)
    {
        return row + "";
    }

    static protected String getColumnImage(int column)
    {
        return COORDINATES.substring(column, column + 1);
    }

    static protected String getCoordinateImage(int column, int row)
    {
    	if (column == 0 && row == 0)
    	{
	        return "pass";
    	}
    	else
    	{
	        return getColumnImage(column) + getRowImage(row);
    	}
    }

    static protected int getColumn(String letter)
    {
        return COORDINATES.indexOf(letter);
    }

    public Color getColor()
    {
        return itsColor;
    }

    public Font getCommentFont()
    {
        return itsCommentFont;
    }

    public Font getCoordinateFont()
    {
        return itsCoordinateFont;
    }

    public Font getStoneNumberFont()
    {
        return itsStoneNumberFont;
    }

    public Font getTextFont()
    {
        return itsTextFont;
    }

    public Font getCommandFont()
    {
        return itsCommandFont;
    }

	public int getSize()
	{
		return itsSize;
	}

	public boolean isShowMultiLineComment()
	{
		return isShowMultiLineComment;
	}

	public boolean isShowOneLineComment()
	{
		return isShowOneLineComment;
	}

	public boolean isShowCoordinates()
	{
		return isShowCoordinates;
	}

    public boolean isShowNumbers()
    {
        return isShowNumbers;
    }

	public boolean isShowCommands()
	{
		return isShowCommands;
	}

	public boolean isShowBoard()
	{
		return isShowBoard;
	}

	public long getRefreshInterval()
	{
		return itsRefreshInterval;
	}
}
