import java.applet.Applet;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Label;
import java.awt.MediaTracker;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.StringTokenizer;
import java.util.Vector;

class Parser
{
    private final String SIZE = "size";
    private final String MOVES = "moves";
    private final String TEXT = "text";
    private final String ONE_LINE_COMMENT = "oneLineComment";
    private final String MULTI_LINE_COMMENT = "multiLineComment";
	private final String SHOW_ONE_LINE_COMMENT = "showOneLineComment";
    private final String SHOW_MULTI_LINE_COMMENT = "showMultiLineComment";
    private final String SHOW_COORDINATES = "showCoordinates";
    private final String SHOW_NUMBERS = "showNumbers";
    private final String SHOW_COMMANDS = "showCommands";
    private final String SHOW_BOARD = "showBoard";
    private final String BG_COLOR = "backgroundColor";
    private final String PASS = "pass";
    private final String URL_ONLINE_GAME = "urlOnlineGame";
    private final String URL_UPDATE_ONLINE_GAME = "urlUpdateOnlineGame";
    private final String REFRESH_INTERVAL = "refreshInterval";

    private Vector  itsMoves               = new Vector();
    private int     itsSize                = Defs.BIG_SIZE;
    private boolean isShowOneLineComment   = true;
    private boolean isShowMultiLineComment = true;
    private boolean isShowCoordinates      = true;
    private boolean isShowNumbers          = true;
    private boolean isShowCommands         = true;
    private boolean isShowBoard            = true;
    private Color   itsColor               = new Color(225, 225, 225);
    private String  itsUrlOnlineGame       = null;
    private String  itsUrlUpdateOnlineGame = null;
    private long    itsRefreshInterval     = 60000; // 60 seconds

    public Parser(Applet applet)
    {
    	parseSize(applet.getParameter(SIZE));
		parseShowOneLineComment(applet.getParameter(SHOW_ONE_LINE_COMMENT));
		parseShowMultiLineComment(applet.getParameter(SHOW_MULTI_LINE_COMMENT));
		parseMoves(applet.getParameter(MOVES));

        String oneLineCommentAll = applet.getParameter(ONE_LINE_COMMENT);

        for (int i = 0; i < itsMoves.size(); i++)
        {
        	int moveNumber = ((Move)itsMoves.elementAt(i)).getNumber();

            String parameter = applet.getParameter(TEXT + moveNumber);

			parseText(parameter, i);

            parameter = applet.getParameter(ONE_LINE_COMMENT + moveNumber);

            if (parameter == null)
            {
            	// Use default comment (if any)
                parameter = oneLineCommentAll;
			}

			parseOneLineComment(parameter, i);

            parameter = applet.getParameter(MULTI_LINE_COMMENT + moveNumber);

			parseMultiLineComment(parameter, i);
		}

		parseShowCoordinates(applet.getParameter(SHOW_COORDINATES));
		parseShowNumbers(applet.getParameter(SHOW_NUMBERS));
		parseShowCommands(applet.getParameter(SHOW_COMMANDS));
		parseShowBoard(applet.getParameter(SHOW_BOARD));
		parseColor(applet.getParameter(BG_COLOR));
		parseUrlOnlineGame(applet.getParameter(URL_ONLINE_GAME));
		parseUrlUpdateOnlineGame(applet.getParameter(URL_UPDATE_ONLINE_GAME));
		parseRefreshInterval(applet.getParameter(REFRESH_INTERVAL));
    }

    private void parseSize(String parameter)
    {
		//System.out.println("parseSize(" + parameter + ")");

		// Applet Size
		try
		{
            if (parameter != null)
            {
				int size = Integer.parseInt(parameter);
	
				if (size == Defs.SMALL_SIZE || size == Defs.BIG_SIZE)
				{
					itsSize = size;
				}
            }
		}
		catch (Exception e)
		{
			//System.out.println("*** Applet Size - Exception : " + e);
		}

		if (itsSize == Defs.SMALL_SIZE)
		{
			isShowOneLineComment   = false;
			isShowMultiLineComment = false;
		}
    }

    private void parseShowOneLineComment(String parameter)
    {
		//System.out.println("parseShowOneLineComment(" + parameter + ")");

		// One Line Comment Box
		try
		{
            if (parameter != null)
            {
				int value = Integer.parseInt(parameter);
				isShowOneLineComment = (value == 1);
            }
		}
		catch (Exception e)
		{
			//System.out.println("*** One Line Comment Box - Exception : " + e);
		}
    }

    private void parseShowMultiLineComment(String parameter)
    {
		//System.out.println("parseShowMultiLineComment(" + parameter + ")");

		// Multiple Line Comment Box
		try
		{
            if (parameter != null)
            {
				int value = Integer.parseInt(parameter);
				isShowMultiLineComment = (value == 1);
            }
		}
		catch (Exception e)
		{
			//System.out.println("*** Multiple Line Comment Box - Exception : " + e);
		}
    }

    public void parseMoves(String parameter)
    {
		//System.out.println("parseMoves(" + parameter + ")");

        // Parse parameter moves
        Vector moves = new Vector();
        Vector variants = new Vector();

        try
        {
            if (parameter != null)
            {
	            int moveNo = 0;
	
	            //System.out.println("moves : '" + parameter + "'");
	
	            StringTokenizer st = new StringTokenizer(parameter);
	
	            while (st.hasMoreTokens())
	            {
	                String str = st.nextToken();
	                
	                //System.out.println("1: str : '" + str + "'");
	
					if (str.equals(PASS))
					{
	                    moves.addElement(new Move(++moveNo, 0, 0));
					}
	                else if (str.length() > 0)
	                {
	                	boolean isVariant = false;
	                	
	                	if (str.startsWith("."))
	                	{
	                		isVariant = true;
	                		str = str.substring(1);
	                		//System.out.println("2: str : '" + str + "'");
	                	}

	                    String letter = str.substring(0, 1);
	                    String number = str.substring(1);
	
	                    int column = Defs.getColumn(letter);
	                    int row = Integer.parseInt(number);

						if (isVariant)
						{
		                    variants.addElement(new Variant(column, row));
						}
						else
						{
		                    moves.addElement(new Move(++moveNo, column, row));
						}
	                }
	            }
            }
        }
        catch (Exception e)
        {
			//System.out.println("*** Parse parameter moves - Exception : " + e);
        }

		if (moves.size() == 0)
		{
			itsMoves.addElement(new Move(1, 8, 8)); // Default
		}
		else
		{
            itsMoves = moves;
        }

        Variant[] allVariants = new Variant[variants.size()];
		variants.copyInto(allVariants);

        ((Move)itsMoves.elementAt(itsMoves.size()-1)).setVariant(allVariants);
    }

    private void parseText(String parameter, int moveNumber)
    {
		//System.out.println("parseText(" + parameter + ", " + moveNumber + ")");

        // Parse parameter text
        try
        {
            if (parameter != null && moveNumber < itsMoves.size())
            {
		        Vector data = new Vector();
	
	            StringTokenizer st = new StringTokenizer(parameter, "|");
	
	            if (st.countTokens() % 2 == 0)
	            {
	                while (st.hasMoreTokens())
	                {
	                    String str1 = st.nextToken().toLowerCase();
	                    String str2 = st.nextToken();
	
	                    if (str1.length() > 0)
	                    {
	                        String letter = str1.substring(0, 1);
	                        String number = str1.substring(1);
	
	                        int column = Defs.getColumn(letter);
	                        int row = Integer.parseInt(number);
	
	                        data.addElement(new Text(column, row, str2));
	                    }
	                }
	            }
	
	            if (data.size() > 0)
	            {
	                Text[] allText = new Text[data.size()];
					data.copyInto(allText);
	
	                ((Move)itsMoves.elementAt(moveNumber)).setText(allText);
	            }
            }
        }
        catch (Exception e)
        {
			//System.out.println("*** Parse parameter text - Exception : " + e);
        }
    }

    private void parseOneLineComment(String parameter, int moveNumber)
    {
		//System.out.println("parseOneLineComment(" + parameter + ", " + moveNumber + ")");

		try
		{
            if (parameter != null && moveNumber < itsMoves.size())
            {
		        ((Move)itsMoves.elementAt(moveNumber)).setOneLineComment(parameter);
            }
		}
		catch (Exception e)
		{
			//System.out.println("*** One Line Comment - Exception : " + e);
		}
    }

    private void parseMultiLineComment(String parameter, int moveNumber)
    {
		//System.out.println("parseMultiLineComment(" + parameter + ", " + moveNumber + ")");

		try
		{
            if (parameter != null && moveNumber < itsMoves.size())
            {
		        // Parse parameter multi line comment
	            Vector data = new Vector();
	
	            int j = 0;
	
	            while (j < parameter.length())
	            {
	                String str;
	
	                int index = parameter.indexOf("\\n", j);
	
	                if (index == -1)
	                {
	                    str = parameter.substring(j);
	                }
	                else
	                {
	                    str = parameter.substring(j, index);
	                }
	
	                data.addElement(str);
	
	                if (index == -1)
	                {
	                    break;
	                }
	                else
	                {
	                    j = index + 2;
	                }
	            }
	
	            if (data.size() > 0)
	            {
	                String[] multiLineComment = new String[data.size()];
					data.copyInto(multiLineComment);

	                ((Move)itsMoves.elementAt(moveNumber)).setMultiLineComment(multiLineComment);
		        }
            }
		}
		catch (Exception e)
		{
			//System.out.println("*** Multi Line Comment - Exception : " + e);
		}
    }

    private void parseShowCoordinates(String parameter)
    {
		//System.out.println("parseShowCoordinates(" + parameter + ")");

        // Coordinates
        try
        {
            if (parameter != null)
            {
	            int value = Integer.parseInt(parameter);
	            isShowCoordinates = (value == 1);
            }
        }
        catch (Exception e)
        {
			//System.out.println("*** Coordinates - Exception : " + e);
        }
    }

    private void parseShowNumbers(String parameter)
    {
		//System.out.println("parseShowNumbers(" + parameter + ")");

        // Numbers
        try
        {
            if (parameter != null)
            {
	            int value = Integer.parseInt(parameter);
	            isShowNumbers = (value == 1);
            }
        }
        catch (Exception e)
        {
			//System.out.println("*** Numbers - Exception : " + e);
        }
    }

    private void parseShowCommands(String parameter)
    {
		//System.out.println("parseShowCommands(" + parameter + ")");

        // Commands Box
        try
        {
            if (parameter != null)
            {
	            int value = Integer.parseInt(parameter);
	            isShowCommands = (value == 1);
            }
        }
        catch (Exception e)
        {
			//System.out.println("*** Commands Box - Exception : " + e);
        }
    }

    private void parseShowBoard(String parameter)
    {
		//System.out.println("parseShowBoard(" + parameter + ")");

        // Board
        try
        {
            if (parameter != null)
            {
	            int value = Integer.parseInt(parameter);
	            isShowBoard = (value == 1);
            }
        }
        catch (Exception e)
        {
			//System.out.println("*** Board - Exception : " + e);
        }
    }

    private void parseColor(String parameter)
    {
		//System.out.println("parseColor(" + parameter + ")");

        // Background Color
        try
        {
            if (parameter != null)
            {
                StringTokenizer st = new StringTokenizer(parameter, "|");

                if (st.countTokens() % 3 == 0)
                {
                    int R = Integer.parseInt(st.nextToken());
                    int G = Integer.parseInt(st.nextToken());
                    int B = Integer.parseInt(st.nextToken());

                    itsColor = new Color(R, G, B);
                }
            }
        }
        catch (Exception e)
        {
			//System.out.println("*** Background Color - Exception : " + e);
            itsColor = new Color(225, 0, 0);
        }
    }

    private void parseRefreshInterval(String parameter)
    {
		//System.out.println("parseRefreshInterval(" + parameter + ")");

		// Refresh Interval
		try
		{
            if (parameter != null)
            {
            	int value = Integer.parseInt(parameter);

	            if (value > 0)
	            {
	            	itsRefreshInterval = value * 1000;
					//System.out.println("--- itsRefreshInterval : " + itsRefreshInterval);
	            }
            }
		}
		catch (Exception e)
		{
			//System.out.println("*** One Line Comment Box - Exception : " + e);
		}
    }

	private void parseUrlOnlineGame(String parameter)
    {
		//System.out.println("parseUrlOnlineGame(" + parameter + ")");
		itsUrlOnlineGame = parameter;
    }

	private void parseUrlUpdateOnlineGame(String parameter)
    {
		//System.out.println("parseUrlUpdateOnlineGame(" + parameter + ")");
    	itsUrlUpdateOnlineGame = parameter;
    }

    public Vector getMoves()
    {
        return itsMoves;
    }

    public int getSize()
    {
        return itsSize;
    }

	public boolean getShowOneLineComment()
	{
		return isShowOneLineComment;
	}

	public boolean getShowMultiLineComment()
	{
		return isShowMultiLineComment;
	}

	public boolean getShowCoordinates()
	{
		return isShowCoordinates;
	}

    public boolean getShowNumbers()
    {
        return isShowNumbers;
    }

    public boolean getShowCommands()
    {
        return isShowCommands;
    }

    public boolean getShowBoard()
    {
        return isShowBoard;
    }

    public Color getColor()
    {
        return itsColor;
    }

    public String getUrlOnlineGame()
    {
        return itsUrlOnlineGame;
    }

    public String getUrlUpdateOnlineGame()
    {
        return itsUrlUpdateOnlineGame;
    }

	public long getRefreshInterval()
	{
		return itsRefreshInterval;
	}
}
