
import java.applet.Applet;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Label;
import java.awt.MediaTracker;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 * RenLib applet.
 *
 * Display a renju position using RenLib generated applet parameter data.
 * The board image and stone images comes from renjuclass.
 *
 * @version 1.4, 2004-03-14 (JDK 1.1.8)
 * @author Frank Arkbo
 */
public class RenLibApplet extends Applet
{
    // The drawing area where to display renju position on a renju board
    private Board   itsBoard;

    // The command buttons
    private Command itsCommand;

    private OneLineComment   itsOneLineComment;    // The comment above the board
    private MultiLineComment itsMultiLineComment;  // The comment on the right side of the board

    private Panel itsPanelBoardCommand      = new Panel();
    private Panel itsPanelBoardMultiComment = new Panel();

    /**
     * Initialize the applet. Load images.
     */
    public void init()
    {
        Parser parser = new Parser(this);

		Defs defs = new Defs(parser);

        setLayout(new BorderLayout());

        itsBoard = new Board(this, parser, defs);
        itsBoard.setBackground(defs.getColor());

        itsCommand = new Command(this, parser, defs, itsBoard);
        itsCommand.setBackground(defs.getColor());

        itsOneLineComment = new OneLineComment(defs);
        itsOneLineComment.setBackground(defs.getColor());

        itsMultiLineComment = new MultiLineComment(defs);

        itsBoard.setCommand(itsCommand);
        itsBoard.setOneLineComment(itsOneLineComment);
        itsBoard.setMultiLineComment(itsMultiLineComment);

        itsBoard.init();

		// Construct panel layout
		itsPanelBoardCommand.setLayout(new BorderLayout());

		itsPanelBoardCommand.add(BorderLayout.CENTER, itsBoard);

		if (defs.isShowCommands())
		{
			itsPanelBoardCommand.add(BorderLayout.SOUTH, itsCommand);
		}

		itsPanelBoardMultiComment.setLayout(new BorderLayout());
		itsPanelBoardMultiComment.setBackground(defs.getColor());

        itsPanelBoardMultiComment.add(BorderLayout.CENTER, itsPanelBoardCommand);

		if (defs.isShowMultiLineComment())
		{
			itsPanelBoardMultiComment.add(BorderLayout.EAST, itsMultiLineComment);
		}

		if (defs.isShowOneLineComment())
		{
			add(BorderLayout.NORTH, itsOneLineComment);
		}

		add(BorderLayout.CENTER, itsPanelBoardMultiComment);
    }

    public void destroy()
    {
		itsCommand.destroy();
        remove(itsCommand);
        remove(itsBoard);
    }

    public String getAppletInfo()
    {
        return "A renju viewer by Frank Arkbo";
    }

    public void start()
    {
        itsCommand.setEnabled(true);
    }

    public void stop()
    {
        itsCommand.stop();
        itsCommand.setEnabled(false);
    }
}
